#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `sho` package."""

import sys
import unittest
from io import StringIO
import webbrowser
#import sho
import logging

logger = logging.info(__name__)


class TestSho(unittest.TestCase):
    """Tests for Sho package."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_simple_import(self):
        """Sample pytest test function with the pytest fixture as an argument."""
        example_string = "test_dd"
        #with Capturing() as output:
        #    sho.w(example_string)
        #logger.info(output)
        #assert output[0] == example_string

    @unittest.skip("Webrowser Error")
    def test_dataframe_as_html(self):
        """Ensure a simple dataframe can be rendered with html"""
        import pandas as pd
        import sho
        df = pd.DataFrame([{"a":"b"}])
        sho.w(df)
        #import seaborn as sns
        #iris_df = sns.load_dataset('iris')
        #sho.w(iris_df)


if __name__ == "__main__":
    unittest.main()


# -*- coding: utf-8 -*-
from typing import Any

from sho.constants import OUTPUT_TYPE
from sho.sho import output, output_detail

"""Top-level package for sho."""

__author__ = """Dave Dawson"""
__email__ = 'davedawson.co@gmail.com'
__version__ = '0.1.0'


def w(obj, output_type: str=OUTPUT_TYPE.DEFAULT.value)->Any:
    """
    Main function called, defaults to
        :param obj: the variable to show
        :param output_type=OUTPUT_TYPE.DEFAULT.value: ability to force output types under certain scenarios
    """

    output(obj,output_type)

def ck(obj, output_type: str=OUTPUT_TYPE.DEFAULT.value)->Any:
    """
    Main function called, defaults to
        :param obj: the variable to show
        :param output_type=OUTPUT_TYPE.DEFAULT.value: ability to force output types under certain scenarios

    """

    output_detail(obj,output_type)



__author__ = "David Dawson"
__copyright__ = "Copyright 2020, David Dawson"
__credits__ = ["David Dawson"]
__license__ = "GPL"
__version__ = "1.0.1"
__maintainer__ = "Dave Dawson"
__email__ = "davedawson.co@gmail.com"
__status__ = "Production"

import logging
import os
import time
import sys
import tempfile
import webbrowser
from typing import Any

import pandas as pd

from pivottablejs import pivot_ui
from sho.constants import OUTPUT_TYPE

logger = logging.getLogger(__name__)

# this is a pointer to the module object instance itself.
this = sys.modules[__name__]

def display_as_string_via_html(obj: Any):
    """
    Display a variable as a default HTML
        :param obj: the object to be displayed.
    """


def display_string_output(obj: Any):
    """
    Display a string
        :param obj: 
    """
    print(obj)

def display_dataframe_with_pivotablejs(obj: Any):
    """
    Function to convert a variable to a pivotable js
        :param obj: table object to display,
    """
    tf = tempfile.NamedTemporaryFile(prefix="sho_", suffix=".html", delete=False)
    file_path = tf.name
    cols = list(obj.columns.values)
    print(f"File Name : {tf.name}")
    pivot_ui(obj, outfile_path=file_path, vals=cols)
    try: 
        browser = webbrowser.get()
        browser.open('file://' + os.path.realpath(file_path))
    except Exception as e:
        logger.info(f"Couldn't find chrome !")
        webbrowser.open('file://' + os.path.realpath(file_path))
    time.sleep(5)
    logger.info(f"File Name : {tf.name}")

def display_dataframe_with_pandas_profiling(obj: Any):
    """
    Function to convert a variable to a pivotable js
        :param obj: table object to display,
    """
    import pandas_profiling
    tf = tempfile.NamedTemporaryFile(prefix="sho_", suffix=".html")
    file_path = tf.name
    profile = obj.profile_report(title='Pandas Profiling Report')
    profile.to_file(output_file=file_path)  
    browser= webbrowser.get('chrome')
    browser.open('file://' + os.path.realpath(file_path))

def display_html_output(obj: Any):
    """
    Function to convert a variable to a pivotable js
        :param obj: table object to display,
    """
    if isinstance(obj, pd.core.frame.DataFrame) :
        display_dataframe_with_pivotablejs(obj)
        # Create more elif's here !
    else:
        display_as_string_via_html(obj)

def get_output_type_for_object(obj: Any):
    """
    If Default selected allow the community to decide the best way to show your variable for you !
        :param obj: variable to display
    """
    return OUTPUT_TYPE.HTML.value

def output( obj: Any, output_type: str):
    """
    Function to map to desired output type
        :param obj: variable to display
        :param output_type: desired output type
    """
    if output_type == OUTPUT_TYPE.DEFAULT.value :
        output_type = get_output_type_for_object(obj)
    
    switcher = {    OUTPUT_TYPE.HTML.value : this.display_html_output,
                    OUTPUT_TYPE.STRING.value : this.display_string_output}
    switcher[output_type](obj)

def output_detail( obj: Any, output_type: str):
    """
    Function to map to desired output type
        :param obj: variable to display
        :param output_type: desired output type
    """
    if output_type == OUTPUT_TYPE.DEFAULT.value :
        output_type = get_output_type_for_object(obj)
    
    switcher = {    OUTPUT_TYPE.HTML.value : this.display_detailed_html_output,
                    OUTPUT_TYPE.STRING.value : this.display_detailed_string_output}
    switcher[output_type](obj)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
"pip==20.2.2",
"bump2version==1.0.0",
"llvmlite==0.34.0",
"wheel==0.35.1",
"watchdog==0.10.2",
"flake8==3.8.1",
"ipython==7.16.0",
"tox==3.19.0",
"coverage==5.1",
"Sphinx==3.2.1",
"twine==3.1.1",
"pandas==1.1.1",
"numpy==1.19.1",
"pivottablejs==0.9.0",
"pandas_profiling==2.8.0",
"seaborn==0.10.1",
"pytest==6.0.1",
"pytest-runner==5.2"
]

setup_requirements = ['pytest-runner', ]

test_requirements = ['unittest>=3', ]

setup(
    author="David Dawson",
    author_email='davedawson.co@gmail.com',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="Visualize python objects in the best way possible",
    install_requires=requirements,
    license="MIT license",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='sho',
    name='sho',
    packages=find_packages(include=['sho', 'sho.*']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/davewd/sho',
    version='0.4.20',
    zip_safe=False,
)
